@@ -28,6 +28,7 @@
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
 #include <xmlsec/dl.h>
+#include <xmlsec/xmltree.h>
 #include <xmlsec/private.h>
 
 #include <xmlsec/mscng/app.h>
@@ -122,6 +123,9 @@ xmlSecCryptoGetFunctions_mscng(void) {
     gXmlSecMSCngFunctions->transformAes128CbcGetKlass           = xmlSecMSCngTransformAes128CbcGetKlass;
     gXmlSecMSCngFunctions->transformAes192CbcGetKlass           = xmlSecMSCngTransformAes192CbcGetKlass;
     gXmlSecMSCngFunctions->transformAes256CbcGetKlass           = xmlSecMSCngTransformAes256CbcGetKlass;
+    gXmlSecMSCngFunctions->transformAes128GcmGetKlass           = xmlSecMSCngTransformAes128GcmGetKlass;
+    gXmlSecMSCngFunctions->transformAes192GcmGetKlass           = xmlSecMSCngTransformAes192GcmGetKlass;
+    gXmlSecMSCngFunctions->transformAes256GcmGetKlass           = xmlSecMSCngTransformAes256GcmGetKlass;
     gXmlSecMSCngFunctions->transformKWAes128GetKlass            = xmlSecMSCngTransformKWAes128GetKlass;
     gXmlSecMSCngFunctions->transformKWAes192GetKlass            = xmlSecMSCngTransformKWAes192GetKlass;
     gXmlSecMSCngFunctions->transformKWAes256GetKlass            = xmlSecMSCngTransformKWAes256GetKlass;
@@ -161,7 +165,7 @@ xmlSecCryptoGetFunctions_mscng(void) {
     gXmlSecMSCngFunctions->transformEcdsaSha256GetKlass         = xmlSecMSCngTransformEcdsaSha256GetKlass;
 #endif /* XMLSEC_NO_SHA256 */
 
-#ifdef XMLSEC_MSCNG_TODO
+#ifndef XMLSEC_NO_SHA384
     gXmlSecMSCngFunctions->transformEcdsaSha384GetKlass         = xmlSecMSCngTransformEcdsaSha384GetKlass;
 #endif /* XMLSEC_NO_SHA384 */
 
@@ -344,7 +348,7 @@ xmlSecMSCngShutdown(void) {
  * Returns: 0 on success or a negative value otherwise.
  */
 int
-xmlSecMSCngGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
+xmlSecMSCngGenerateRandom(xmlSecBufferPtr buffer, xmlSecSize size) {
     NTSTATUS status;
     int ret;
 
@@ -353,14 +357,14 @@ xmlSecMSCngGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
 
     ret = xmlSecBufferSetSize(buffer, size);
     if(ret < 0) {
-	xmlSecInternalError2("xmlSecBufferSetSize", NULL, "size=%d", size);
+    xmlSecInternalError2("xmlSecBufferSetSize", NULL, "size=%d", size);
         return(-1);
     }
 
     status = BCryptGenRandom(
         NULL,
         (PBYTE)xmlSecBufferGetData(buffer),
-        size,
+        (ULONG)size,
         BCRYPT_USE_SYSTEM_PREFERRED_RNG);
     if(status != STATUS_SUCCESS) {
         xmlSecMSCngNtError("BCryptGenRandom", NULL, status);
@@ -406,37 +410,6 @@ xmlSecMSCngKeysMngrInit(xmlSecKeysMngrPtr mngr) {
     return(0);
 }
 
-
-LPWSTR
-xmlSecMSCngMultiByteToWideChar(const char* multiByte) {
-    LPWSTR wideChar = NULL;
-    int size;
-    int ret;
-
-    xmlSecAssert2(multiByte != NULL, NULL);
-
-    /* determinze size */
-    size = MultiByteToWideChar(CP_ACP, 0, multiByte, -1, NULL, 0);
-    if(size < 0) {
-        return(NULL);
-    }
-
-    wideChar = (LPWSTR)xmlMalloc(size * sizeof(WCHAR));
-    if(wideChar == NULL) {
-        xmlSecMallocError(size * sizeof(WCHAR), NULL);
-        return(NULL);
-    }
-
-    /* process the input string */
-    ret = MultiByteToWideChar(CP_ACP, 0, multiByte, -1, wideChar, size);
-    if(ret < 0) {
-        xmlFree(wideChar);
-        return(NULL);
-    }
-
-    return(wideChar);
-}
-
 /**
  * xmlSecMSCngConvertUtf8ToTstr:
  * @str:         the string to convert.
@@ -447,12 +420,7 @@ xmlSecMSCngMultiByteToWideChar(const char* multiByte) {
  */
 LPTSTR
 xmlSecMSCngConvertUtf8ToTstr(const xmlChar* str) {
-#ifdef UNICODE
-    return xmlSecMSCngConvertUtf8ToUnicode(str);
-#else  /* UNICODE */
-    xmlSecNotImplementedError(NULL);
-    return(NULL);
-#endif /* UNICODE */
+    return(xmlSecWin32ConvertUtf8ToTstr(str));
 }
 
 /**
@@ -465,12 +433,7 @@ xmlSecMSCngConvertUtf8ToTstr(const xmlChar* str) {
  */
 xmlChar*
 xmlSecMSCngConvertTstrToUtf8(LPCTSTR str) {
-#ifdef UNICODE
-    return xmlSecMSCngConvertUnicodeToUtf8(str);
-#else  /* UNICODE */
-    xmlSecNotImplementedError(NULL);
-    return(NULL);
-#endif /* UNICODE */
+    return(xmlSecWin32ConvertTstrToUtf8(str));
 }
 
 /**
@@ -483,35 +446,7 @@ xmlSecMSCngConvertTstrToUtf8(LPCTSTR str) {
  */
 xmlChar*
 xmlSecMSCngConvertUnicodeToUtf8(LPCWSTR str) {
-    xmlChar * res = NULL;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    /* call WideCharToMultiByte first to get the buffer size */
-    ret = WideCharToMultiByte(CP_UTF8, 0, str, -1, NULL, 0, NULL, NULL);
-    if(ret <= 0) {
-        return(NULL);
-    }
-    len = ret + 1;
-
-    /* allocate buffer */
-    res = (xmlChar*)xmlMalloc(sizeof(xmlChar) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(xmlChar) * len, NULL);
-        return(NULL);
-    }
-
-    /* convert */
-    ret = WideCharToMultiByte(CP_UTF8, 0, str, -1, (LPSTR)res, len, NULL, NULL);
-    if(ret <= 0) {
-        xmlFree(res);
-        return(NULL);
-    }
-
-    /* done */
-    return(res);
+    return(xmlSecWin32ConvertUnicodeToUtf8(str));
 }
 
 /**
@@ -524,31 +459,5 @@ xmlSecMSCngConvertUnicodeToUtf8(LPCWSTR str) {
  */
 LPWSTR
 xmlSecMSCngConvertUtf8ToUnicode(const xmlChar* str) {
-    LPWSTR res;
-    int len;
-    int ret;
-
-    xmlSecAssert2(str != NULL, NULL);
-
-    ret = MultiByteToWideChar(CP_UTF8, 0, (LPCCH)str, -1, NULL, 0);
-    if(ret == 0) {
-        xmlSecMSCngLastError("MultiByteToWideChar", NULL);
-        return(NULL);
-    }
-    len = ret + 1;
-
-    res = (LPWSTR)xmlMalloc(sizeof(WCHAR) * len);
-    if(res == NULL) {
-        xmlSecMallocError(sizeof(WCHAR) * len, NULL);
-        return(NULL);
-    }
-
-    ret = MultiByteToWideChar(CP_UTF8, 0, (LPCCH)str, -1, res, len);
-    if(ret == 0) {
-        xmlSecMSCngLastError("MultiByteToWideChar", NULL);
-        xmlFree(res);
-        return(NULL);
-    }
-
-    return(res);
+    return(xmlSecWin32ConvertUtf8ToUnicode(str));
 }
