@@ -785,8 +785,12 @@ xmlSecMSCngKeyDataX509VerifyAndExtractKey(xmlSecKeyDataPtr data,
         }
 
         if((keyInfoCtx->keyReq.keyType & xmlSecKeyDataTypePrivate) != 0) {
-            xmlSecNotImplementedError(NULL);
-            return(-1);
+            keyValue = xmlSecMSCngCertAdopt(certCopy, xmlSecKeyDataTypePrivate);
+            if(keyValue == NULL) {
+                xmlSecInternalError("xmlSecMSCngCertAdopt",
+                    xmlSecKeyDataGetName(data));
+                return(-1);
+            }
         } else if((keyInfoCtx->keyReq.keyType & xmlSecKeyDataTypePublic) != 0) {
             keyValue = xmlSecMSCngCertAdopt(certCopy, xmlSecKeyDataTypePublic);
             if(keyValue == NULL) {
@@ -933,9 +937,9 @@ xmlSecMSCngX509NameWrite(PCERT_NAME_BLOB nm) {
         return(NULL);
     }
 
-    res = xmlSecMSCngConvertTstrToUtf8(resT);
+    res = xmlSecWin32ConvertTstrToUtf8(resT);
     if (NULL == res) {
-        xmlSecInternalError("xmlSecMSCngConvertTstrToUtf8", NULL);
+        xmlSecInternalError("xmlSecWin32ConvertTstrToUtf8", NULL);
         xmlFree(resT);
         return(NULL);
     }
